#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import typer
import requests
import os
import zipfile
from subprocess import check_output
from subprocess import STDOUT
from subprocess import call
from subprocess import Popen
from pathlib import Path
from sys import platform
import shutil
import time

def download_and_unarchive(url, download_path, extract_dir = os.path.join(os.path.expanduser('~'), ".mft/")):

  response = requests.get(url, stream=True)
  file_size = int(response.headers['Content-Length'])
  with typer.progressbar(length=file_size) as progress:
    with open(download_path, "wb") as handle:
      for data in response.iter_content(chunk_size=8192 * 2):
        progress.update(len(data))
        handle.write(data)

  print("Un archiving ....")
  with zipfile.ZipFile(download_path,"r") as zip_ref:
    zip_ref.extractall(extract_dir)

  os.remove(download_path)

def restart_service(bin_path, daemon_script_name):
  current_dir =  os.getcwd()
  try:
    os.chdir(bin_path)
    os.chmod(daemon_script_name, 0o744)
    rc = call(["./" + daemon_script_name, "stop"])
    rc = call(["./" + daemon_script_name, "start"])
  finally:
    os.chdir(current_dir)

def stop_service(bin_path, daemon_script_name):
  current_dir =  os.getcwd()
  try:
    os.chdir(bin_path)
    os.chmod(daemon_script_name, 0o744)
    rc = call(["./" + daemon_script_name, "stop"])
  finally:
    os.chdir(current_dir)


def validate_java_availability(required_version):
  """
      Issue 96: https://github.com/apache/airavata-mft/issues/96

      References:
      ---------
      https://stackoverflow.com/questions/31807882/get-java-version-number-from-python
      https://stackoverflow.com/questions/1332598/how-to-determine-whether-java-is-installed-on-a-system-through-python
      https://stackoverflow.com/questions/74206258/how-are-oracle-jdk-versions-numbered
      https://docs.python.org/3.8/library/subprocess.html#subprocess.check_output
      https://stackoverflow.com/questions/2411288/java-versioning-and-terminology-1-6-vs-6-0-openjdk-vs-sun
  """
  if shutil.which("java"):
    res = check_output(['java', '-version'], stderr=STDOUT).decode('utf-8')
    """
      res will have the value similar to the following
      openjdk version "17.0.5" 2022-10-18 LTS
      OpenJDK Runtime Environment Corretto-17.0.5.8.1 (build 17.0.5+8-LTS)
      OpenJDK 64-Bit Server VM Corretto-17.0.5.8.1 (build 17.0.5+8-LTS, mixed mode, sharing)
    """
    java_version = ''
    count = 0
    for c in res:
      if c == '"' or count == 1:
        if (c == '.' or c == '"') and count == 1:
          break
        if count == 0:
          count += 1
          continue
        java_version += c

    java_version = int(java_version)
    if java_version < required_version:
      print("Airavata MFT requires Java version " + required_version + " or higher")
      print("If you have more than one version of java please set java version "+ required_version +" or higher to the path")
      raise typer.Exit()
  else:
    print("Java is either not installed or path hasn't been set properly")
    raise typer.Exit()


def start_mft():
  print("Setting up MFT Services")

  required_java_version = 11
  if platform == "linux" or platform == "linux2":
    consul_url = "https://releases.hashicorp.com/consul/1.7.1/consul_1.7.1_linux_amd64.zip"
    validate_java_availability(required_java_version)
  elif platform == "darwin":
    consul_url = "https://releases.hashicorp.com/consul/1.7.1/consul_1.7.1_darwin_amd64.zip"
    validate_java_availability(required_java_version)
  elif platform == "win32":
    print("Windows support is not available yet")
    raise typer.Exit()
  else:
    print("Un supported platform: " + platform)
    raise typer.Exit()

  mft_dir = os.path.join(os.path.expanduser('~'), ".mft")
  if not os.path.exists(mft_dir):
    os.makedirs(mft_dir)

  path = os.path.join(os.path.expanduser('~'), ".mft/consul")
  if not os.path.exists(path):
    print("Downloading Consul...")
    zip_path = os.path.join(os.path.expanduser('~'), ".mft/consul.zip")
    download_and_unarchive(consul_url, zip_path, os.path.join(os.path.expanduser('~'), ".mft/"))

  current_dir =  os.getcwd()
  try:
    os.chdir(os.path.join(os.path.expanduser('~'), ".mft"))
    os.chmod("consul", 0o744)

    if os.path.exists("consul.pid"):
      pid = Path('consul.pid').read_text()
      call(["kill", "-9", pid])

    consul_process = Popen(['nohup', './consul', "agent", "-dev"],
                     stdout=open('consul.log', 'w'),
                     stderr=open('consul.err.log', 'a'),
                     preexec_fn=os.setpgrp)

    print("Consul process id: " + str(consul_process.pid))
    with open("consul.pid", "w") as consul_pid:
      consul_pid.write(str(consul_process.pid))
  finally:
    os.chdir(current_dir)

  path = os.path.join(os.path.expanduser('~'), ".mft/Standalone-Service-0.01")
  if not os.path.exists(path):
    url = "https://github.com/apache/airavata-mft/releases/download/v0.0.1/Standalone-Service-0.01-bin.zip"
    print("Downloading MFT Server...")
    zip_path = os.path.join(os.path.expanduser('~'), ".mft/Standalone-Service-0.01-bin.zip")
    download_and_unarchive(url, zip_path)

  restart_service(path + "/bin", "standalone-service-daemon.sh")

  print("MFT Started")


def stop_mft():
  print("Stopping MFT Services")

  path = os.path.join(os.path.expanduser('~'), ".mft/consul")
  if os.path.exists(path):
    current_dir =  os.getcwd()
    try:
      os.chdir(os.path.join(os.path.expanduser('~'), ".mft"))
      os.chmod("consul", 0o744)

      if os.path.exists("consul.pid"):
        pid = Path('consul.pid').read_text()
        call(["kill", "-9", pid])
    finally:
      os.chdir(current_dir)

  path = os.path.join(os.path.expanduser('~'), ".mft/Standalone-Service-0.01")
  if os.path.exists(path):
    stop_service(path + "/bin", "standalone-service-daemon.sh")

  print("MFT Stopped....")

def update_mft():
  stop_mft()

  mft_dir = os.path.join(os.path.expanduser('~'), ".mft")
  if os.path.exists(mft_dir):
    print("Removing .mft directory")
    shutil.rmtree(mft_dir)

  database = os.path.join(os.path.expanduser('~'), "mft_db.mv.db")
  if os.path.exists(database):
    os.remove(database)
  start_mft()

def print_log():
  log_file_path = os.path.join(os.path.expanduser('~'), ".mft", "Standalone-Service-0.01", "logs", "airavata.log")
  log_file = open(log_file_path,"r")
  lines = follow_file(log_file)
  for line in lines:
    print(line)

def follow_file(file):
  #file.seek(0, os.SEEK_END)

  while True:
    line = file.readline()
    if not line:
      time.sleep(0.1)
      continue

    yield line





#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from rich import print
from pick import pick
import typer
from airavata_mft_sdk import mft_client
from airavata_mft_sdk.scp import SCPCredential_pb2
from airavata_mft_sdk.scp import SCPStorage_pb2
from airavata_mft_sdk import MFTTransferApi_pb2
from airavata_mft_sdk import MFTAgentStubs_pb2
from airavata_mft_sdk.common import StorageCommon_pb2
import configparser
import os
import sys
sys.path.append('../airavata_mft_cli')
from airavata_mft_cli import config as configcli

def handle_add_storage():

    private_key_file = typer.prompt("Private Key File Location")

    with open(private_key_file, 'r') as file:
        private_key = file.read()

    public_key_file = typer.prompt("Public Key File Location")
    with open(public_key_file, 'r') as file:
        public_key = file.read()

    passphrase = ""
    is_pass = typer.confirm("Is there a passphrase to Private Key?", False)
    if is_pass:
        passphrase = typer.prompt("Passphrase to Private Key", hide_input=True)
    host_name = typer.prompt("Hostname / IP")
    user_name = typer.prompt("User Name:")
    storage_name = typer.prompt("Storage Name", host_name)

    client = mft_client.MFTClient(transfer_api_port = configcli.transfer_api_port,
                                  transfer_api_secured = configcli.transfer_api_secured,
                                  resource_service_host = configcli.resource_service_host,
                                  resource_service_port = configcli.resource_service_port,
                                  resource_service_secured = configcli.resource_service_secured,
                                  secret_service_host = configcli.secret_service_host,
                                  secret_service_port = configcli.secret_service_port)

    secret_create_req = SCPCredential_pb2.SCPSecretCreateRequest(privateKey=private_key, publicKey=public_key, passphrase = passphrase, user=user_name)
    created_secret = client.scp_secret_api.createSCPSecret(secret_create_req)

    scp_storage_create_req = SCPStorage_pb2.SCPStorageCreateRequest(
        host=host_name, port=22, name=storage_name)

    created_storage = client.scp_storage_api.createSCPStorage(scp_storage_create_req)

    secret_for_storage_req = StorageCommon_pb2.SecretForStorage(storageId = created_storage.storageId,
                                       secretId = created_secret.secretId,
                                       storageType = StorageCommon_pb2.StorageType.SCP)

    client.common_api.registerSecretForStorage(secret_for_storage_req)

    print("Successfully added the SCP endpoint...")


#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import typer
from pick import pick
import airavata_mft_cli.storage.s3 as s3
import airavata_mft_cli.storage.scp as scp
import airavata_mft_cli.storage.azure as azure
import airavata_mft_cli.storage.gcs as gcs
import airavata_mft_cli.storage.local as local
import airavata_mft_cli.storage.swift as swift
import airavata_mft_cli.storage.http as http
from airavata_mft_sdk import mft_client
from airavata_mft_sdk.common import StorageCommon_pb2
from rich.console import Console
from rich.table import Table
from rich import print
import sys
sys.path.append('../airavata_mft_cli')
from airavata_mft_cli import config as configcli
from airavata_mft_cli.util import exception_handler

app = typer.Typer(pretty_exceptions_show_locals=True)

@app.command("add")
def add_storage():
    try:
        title = "Select storage type: "
        options = ["S3", "Google Cloud Storage (GCS)", "Azure Storage", "Openstack SWIFT", "SCP", "FTP", "Box", "DropBox", "OData", "Agent", "HTTP" ]
        option, index = pick(options, title, indicator="=>")
        if option == "S3":
            s3.handle_add_storage()
        elif option == "Azure Storage":
            azure.handle_add_storage()
        elif option == "Google Cloud Storage (GCS)":
            gcs.handle_add_storage()
        elif option == "Agent":
            local.handle_add_storage()
        elif option == "Openstack SWIFT":
            swift.handle_add_storage()
        elif option == "SCP":
            scp.handle_add_storage()
        elif option == "HTTP":
            http.handle_add_storage()
    except Exception as e:
        exception_handler(e)

@app.command("remove")
def remove_storage(storage_id):
    client = mft_client.MFTClient(transfer_api_port = configcli.transfer_api_port,
                                        transfer_api_secured = configcli.transfer_api_secured,
                                        resource_service_host = configcli.resource_service_host,
                                        resource_service_port = configcli.resource_service_port,
                                        resource_service_secured = configcli.resource_service_secured,
                                        secret_service_host = configcli.secret_service_host,
                                        secret_service_port = configcli.secret_service_port)
    delete_request = StorageCommon_pb2.SecretForStorageDeleteRequest(storageId=storage_id)
    delete_response = client.common_api.deleteSecretsForStorage(delete_request)
    console = Console()
    console.print("Storage removed: " + str(delete_response.status))

@app.command("list")
def list_storage():
    try:
        client = mft_client.MFTClient(transfer_api_port = configcli.transfer_api_port,
                                    transfer_api_secured = configcli.transfer_api_secured,
                                    resource_service_host = configcli.resource_service_host,
                                    resource_service_port = configcli.resource_service_port,
                                    resource_service_secured = configcli.resource_service_secured,
                                    secret_service_host = configcli.secret_service_host,
                                    secret_service_port = configcli.secret_service_port)
        list_req = StorageCommon_pb2.StorageListRequest()
        list_response = client.common_api.listStorages(list_req)

        console = Console()
        table = Table(show_header=True, header_style='bold #2070b2')

        table.add_column('Storage Name', justify='left')
        table.add_column('Type', justify='center')
        table.add_column('Storage ID', justify='center')

        for storage in list_response.storageList:

            table.add_row('[bold]' + storage.storageName + '[/bold]',
                        StorageCommon_pb2.StorageType.Name(storage.storageType),
                        storage.storageId)

        console.print(table)
    except Exception as e:
        exception_handler(e)


#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from rich import print
from pick import pick
import typer
from airavata_mft_sdk import mft_client
from airavata_mft_sdk.http import HttpCredential_pb2
from airavata_mft_sdk.http import HTTPStorage_pb2
from airavata_mft_sdk import MFTTransferApi_pb2
from airavata_mft_sdk import MFTAgentStubs_pb2
from airavata_mft_sdk.common import StorageCommon_pb2
import configparser
import os
import sys
sys.path.append('../airavata_mft_cli')
from airavata_mft_cli import config as configcli

def handle_add_storage():

    client = mft_client.MFTClient(transfer_api_port = configcli.transfer_api_port,
                                      transfer_api_secured = configcli.transfer_api_secured,
                                      resource_service_host = configcli.resource_service_host,
                                      resource_service_port = configcli.resource_service_port,
                                      resource_service_secured = configcli.resource_service_secured,
                                      secret_service_host = configcli.secret_service_host,
                                      secret_service_port = configcli.secret_service_port)

    base_url = typer.prompt("Base URL")
    storage_name = typer.prompt("Storage Name", base_url)

    options = ["Basic Auth", "Token" ]
    option, index = pick(options, "What is the authentication method", indicator="=>")

    if index == 0:
        user_name = typer.prompt("User Name")
        password = typer.prompt("Password")
        basic_auth = HttpCredential_pb2.BasicAuth(userName=user_name, password=password)
        http_secret = HttpCredential_pb2.HTTPSecret(basic=basic_auth)
    elif index == 1:
        token = typer.prompt("Token")
        token_auth = HttpCredential_pb2.TokenAuth(accessToken=token)
        http_secret = HttpCredential_pb2.HTTPSecret(token=token_auth)

    secret_create_req= HttpCredential_pb2.HTTPSecretCreateRequest(secret=http_secret)
    created_secret = client.http_secret_api.createHTTPSecret(secret_create_req)

    http_storage_create_req = HTTPStorage_pb2.HTTPStorageCreateRequest(
            baseUrl=base_url, name=storage_name)

    created_storage = client.http_storage_api.createHTTPStorage(http_storage_create_req)

    secret_for_storage_req = StorageCommon_pb2.SecretForStorage(storageId = created_storage.storageId,
                                           secretId = created_secret.secretId,
                                           storageType = StorageCommon_pb2.StorageType.HTTP)

    client.common_api.registerSecretForStorage(secret_for_storage_req)

    print("Successfully added the HTTP endpoint...")






transfer_api_port = 7003
transfer_api_secured = False
resource_service_host = "localhost"
resource_service_port = 7003
resource_service_secured = False
secret_service_host = "localhost"
secret_service_port = 7003